#ifndef _COURSE_CPP
#define _COURSE_CPP

#include <string>
#include <fstream>
#include "Course.h"

using namespace std;

//-----------------------------------------------------------------
//                            
//-----------------------------------------------------------------
Course::Course( string aCode, string aTitle )
{
   if ( aCode.length() != cCodeSz-1 &&
        (aCode.length() != 0 || aTitle.length() != 0) )
      throw CourseXptn( CourseKey(""), "Course",
                        CourseXptn::keyLen, aCode.c_str() );
   cCode = CourseKey( aCode );
   strncpy( cTitle, aTitle.c_str(), cTitleSz-1 );
                                        cTitle[cTitleSz-1] = '\0';
   cFSem = 0;
   cCompuls = false;
   cCateg[0] = '\0';
   cWH = 0;
   cUnits = 0;
   cPrereq = CourseKey( "" );  //   cPrereq.s[0] = '\0';
   cNoOfStudents = 0;
} // Course::Course

//-----------------------------------------------------------------
//                             
//-----------------------------------------------------------------
void Course::setCode( string aCode )
{
   if ( aCode.length() != cCodeSz-1 )
      throw CourseXptn( CourseKey(cCode), "setCode",
                        CourseXptn::keyLen, aCode.c_str() );
   if ( CourseKey(aCode) == cPrereq )
      throw CourseXptn( CourseKey(cCode), "setCode",
                        CourseXptn::autoRef, aCode.c_str() );
   strcpy( cCode.s, aCode.c_str() );
//   cCode = CourseKey( aCode.c_str() );
} // Course::setCode

void Course::setTitle( string aTitle )
{
   strncpy( cTitle, aTitle.c_str(), cTitleSz-1 );
   cTitle[cTitleSz-1] = '\0';
} // Course::setTitle

void Course::setFSem( int aFSem )
{
   if ( aFSem < 1 || 8 < aFSem )
      throw CourseXptn( cCode, "setFSem", CourseXptn::rangeError,
                        aFSem );
   cFSem = aFSem;
} // Course::setFSem

void Course::setSector( char aSector )
{
   if ( aSector != '' && aSector != '' &&
        aSector != '' && aSector != '' && aSector != '' )
      throw CourseXptn( cCode, "setSector",
                        CourseXptn::noSuchSector, aSector );
   cSector = aSector;
} // Course::setSector

void Course::setCateg( string aCateg )
{
   if ( aCateg != "" && aCateg != "" &&
        aCateg != "" && aCateg != "" )
      throw CourseXptn( cCode, "setCateg",
                        CourseXptn::noSuchCateg, aCateg.c_str() );
   strcpy( cCateg, aCateg.c_str() );
} // Course::setSector

void Course::setWH( int aWH )
{
   if ( aWH <= 0 )
      throw CourseXptn( cCode, "setWH",
                        CourseXptn::rangeError, aWH );
   cWH = aWH;
} // Course::setWH

void Course::setUnits( int aUnits )
{
   if ( aUnits <= 0 )
      throw CourseXptn( cCode, "setUnits",
                        CourseXptn::rangeError, aUnits );
   cUnits = aUnits;
} // Course::setUnits

void Course::delete1Student()
{
   if ( cNoOfStudents <= 0 )
      throw CourseXptn( cCode, "delete1Student",
                        CourseXptn::noStudent );
   --cNoOfStudents;
} // Course::delete1Student

void Course::setPrereq( const string& prCode )
{
   if ( prCode.length() != cCodeSz-1 && prCode.length() != 0 )
      throw CourseXptn( cCode, "setPrereq", CourseXptn::keyLen,
                        prCode.c_str() );
//   if ( cCode.s[0]!='\0' && CourseKey(prCode)==cCode )
   if ( cCode != CourseKey("") && CourseKey(prCode) == cCode )
      throw CourseXptn( cCode, "setPrereq", CourseXptn::autoRef,
                        prCode.c_str() );
//   strcpy( cPrereq.s, prCode.c_str() );
   cPrereq = CourseKey( prCode );
} // Course::setPrereq

void Course::save( ostream& bout ) const
{
   if ( bout.fail() )
      throw CourseXptn( cCode, "save", CourseXptn::fileNotOpen );
   bout.write( cCode.s, sizeof(cCode.s) );    //  
   bout.write( cTitle, sizeof(cTitle) );       //  
   bout.write( reinterpret_cast<const char*>(&cFSem),
               sizeof(cFSem) );                  //  
   bout.write( reinterpret_cast<const char*>(&cCompuls),
               sizeof(cCompuls) );       //   
   bout.write( &cSector, sizeof(cSector) );              // 
   bout.write( cCateg, sizeof(cCateg) );              // 
   bout.write( reinterpret_cast<const char*>(&cWH), sizeof(cWH) );
                                              //   
   bout.write( reinterpret_cast<const char*>(&cUnits),
               sizeof(cUnits) );             //  
   bout.write( cPrereq.s, sizeof(cPrereq.s) );   // 
   bout.write( reinterpret_cast<const char*>(&cNoOfStudents),
               sizeof(cNoOfStudents) );
                                      // .   
   if ( bout.fail() )
      throw CourseXptn( cCode, "save", CourseXptn::cannotWrite );
} // Course::save

void Course::load( istream& bin )
{
   Course tmp;
   bin.read( tmp.cCode.s, cCodeSz );            //  
   if ( !bin.eof() )
   {
      bin.read( tmp.cTitle, cTitleSz );            //  
      bin.read( reinterpret_cast<char*>(&tmp.cFSem), sizeof(cFSem) );
                                                 //  
      bin.read( reinterpret_cast<char*>(&tmp.cCompuls),
                                               sizeof(cCompuls) );
                                         //   
      bin.read( &tmp.cSector, sizeof(cSector) );             // 
      bin.read( tmp.cCateg, cCategSz );                    // 
      bin.read( reinterpret_cast<char*>(&tmp.cWH), sizeof(cWH) );
                                              //   
      bin.read( reinterpret_cast<char*>(&tmp.cUnits), sizeof(cUnits));
                                             //  
      bin.read( tmp.cPrereq.s, cCodeSz );          // 
      bin.read( reinterpret_cast<char*>(&tmp.cNoOfStudents),
                sizeof(cNoOfStudents) ); // . .  
      if ( bin.fail() )
         throw CourseXptn( cCode, "load", CourseXptn::cannotRead );
      *this = tmp;
   } // if ( !bin.eof() ). . .
} // Course::load

void Course::display( ostream& tout ) const
{
   if ( tout.fail() )
      throw CourseXptn( cCode, "display", CourseXptn::fileNotOpen );
   tout << cCode.s << '\t' << cTitle << '\t';
   if ( cCompuls )  tout << "\t";
              else  tout << "\t";
   tout << cSector << '\t' << cCateg << '\t' << cWH << '\t'
        << cUnits << '\t' << cPrereq.s << '\t' << cFSem << '\t'
        << cNoOfStudents << endl;
   if ( tout.fail() )
      throw CourseXptn( cCode, "display", CourseXptn::cannotWrite );
} // Course::display

#endif // _COURSE_CPP
